# Generated by roxygen2: do not edit by hand

S3method(print,bigannoy_index)
export(annoy_build_bigmatrix)
export(annoy_close_index)
export(annoy_is_loaded)
export(annoy_load_bigmatrix)
export(annoy_open_index)
export(annoy_search_bigmatrix)
export(annoy_validate_index)
export(benchmark_annoy_bigmatrix)
export(benchmark_annoy_recall_suite)
export(benchmark_annoy_volume_suite)
export(benchmark_annoy_vs_rcppannoy)
importFrom(Rcpp,evalCpp)
useDynLib(bigANNOY, .registration = TRUE)
